package net.sf.itcb.common.client.aop;

import net.sf.itcb.common.client.exceptions.ClientItcbException;
import net.sf.itcb.common.client.exceptions.ClientItcbExceptionMappingErrors;

import org.aopalliance.intercept.MethodInterceptor;
import org.aopalliance.intercept.MethodInvocation;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.ws.client.core.support.WebServiceGatewaySupport;
import org.springframework.ws.client.support.interceptor.ClientInterceptor;
import org.springframework.ws.soap.client.SoapFaultClientException;

/**
 * This interceptor allows to use generic client in order to call a spring webservice.
 * It can manage each interface which respects those rules :
 * <ul>
 * <li>Only one parameter <b>*Request</b> generated by JAXB using an XSD file. This object contains all information needed by the service</li>
 * <li>A return object <b>*Response</b> generated by JAXB using an XSD file</li>
 * </ul>
 * If it doesn't match, a {@link net.sf.itcb.common.client.exceptions.ClientItcbException} is thrown with {@link net.sf.itcb.common.client.exceptions.ClientExceptionMappingErrors#}.
 *
 * @author Pierre Le Roux
 */
public class ItcbClientInterceptor extends WebServiceGatewaySupport implements MethodInterceptor, ApplicationContextAware {

	protected org.springframework.context.MessageSource messageSource;
	
	@Override
	public void setApplicationContext(ApplicationContext applicationContext)
			throws BeansException {
		messageSource=applicationContext.getBean("common-clientResources", org.springframework.context.MessageSource.class);
	}
	
	
	
	
	@Override
	protected void initGateway() throws Exception {
		for (ClientInterceptor clientInterceptor : this.getInterceptors()) {
			if(clientInterceptor instanceof ClientInterceptorWebserviceTemplateAware) {
				((ClientInterceptorWebserviceTemplateAware)clientInterceptor).setWebserviceTemplate(getWebServiceTemplate());
			}
		}
	}




	public Object invoke(MethodInvocation arg0) throws Throwable {
		if(arg0.getArguments().length != 1) {
			throw new ClientItcbException(ClientItcbExceptionMappingErrors.COMMON_CLIENT_NOT_CORRECT_INTERFACE, messageSource.getMessage("common-client.exception.not_correct_interface", new Object[] {arg0.getMethod()}, LocaleContextHolder.getLocale()));
		}
		try  {
			return getWebServiceTemplate().marshalSendAndReceive(arg0.getArguments()[0]); 
		}
		catch (SoapFaultClientException sfce) { 
			logger.error(sfce.getFaultCode(), sfce);
			throw new ClientItcbException(ClientItcbExceptionMappingErrors.COMMON_CLIENT_TECHNICAL_ERROR, messageSource.getMessage("common-client.exception.technical_error", new Object[] {arg0.getMethod(), getWebServiceTemplate().getDestinationProvider().getDestination(), sfce.getMessage()}, LocaleContextHolder.getLocale()));
		}
		catch(java.lang.IllegalArgumentException iae) {
			logger.error(iae.getMessage(), iae);
			throw new ClientItcbException(ClientItcbExceptionMappingErrors.COMMON_CLIENT_TECHNICAL_ERROR, messageSource.getMessage("common-client.exception.technical_error", new Object[] {arg0.getMethod(), getWebServiceTemplate().getDestinationProvider().getDestination(), iae.getMessage()}, LocaleContextHolder.getLocale()));
		}
		
	}


	
}
